// This test driver program reads an encoded record from a file 
// and decodes it and displays the results..

#include <stdio.h>
#include <stdlib.h>
#include "H323-MESSAGES.h"
#include "rtxsrc/rtxDiag.h"

#define MAXMSGLEN 1024

int main (int argc, char** argv)
{
   OSOCTET	msgbuf[MAXMSGLEN];
   OSBOOL       aligned = TRUE, trace = TRUE;
   FILE*        fp;
   const char*  filename = "message.dat";
   int		i, len, stat;

   // Process command line arguments

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-a")) aligned = TRUE;
         else if (!strcmp (argv[i], "-u")) aligned = FALSE;
         else if (!strcmp (argv[i], "-v")) rtxSetGlobalDiag (TRUE);
         else if (!strcmp (argv[i], "-i")) filename = argv[++i];
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: reader -a | -u [ -v ] [ -i <filename>\n");
            printf ("   -a  use PER aligned encoding\n");
            printf ("   -u  use PER unaligned encoding\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -i <filename>  read encoded msg from <filename>\n");
            printf ("   -notrace  do not display trace info\n");
            return 0;
         }
      }
   }

   // Read input file into a memory buffer

   if (fp = fopen (filename, "rb")) {
      len = fread (msgbuf, 1, sizeof(msgbuf), fp);
   }
   else {
      printf ("Error opening %s for read access\n", filename);
      return -1;
   }

   // Decode

   ASN1PERDecodeBuffer decodeBuffer (msgbuf, len, aligned);
   ASN1T_H225RasMessage msgData;
   ASN1C_H225RasMessage rasMessage (decodeBuffer, msgData);

   decodeBuffer.setTrace (trace);

   stat = rasMessage.Decode ();

   if (trace) {
      printf ("Dump of decoded bit fields:\n");
      decodeBuffer.binDump ("rasMessage");
      printf ("\n");
   }

   if (stat == 0) {
      if (trace) {
         printf ("decode of RasMessage was successful\n");
         rasMessage.Print ("rasMessage");
      }
   }
   else {
       printf ("decode of RasMessage failed\n");
       decodeBuffer.printErrorInfo ();
       return -1;
   }

   return 0;
}
